import functools

import ujson

from bedlam.cache import map_route
from bedlam.errors import BedlamError
from bedlam.request import Request


def error_handler(f):
    @functools.wraps(f)
    def wrap(environ, start_response):
        try:
            return f(environ, start_response)
        except BedlamError as err:
            return err.handle(start_response)
        except:
            return BedlamError.handle(start_response)

    return wrap


@error_handler
def run(environ, start_response):
    request = Request(environ)

    handler, groups, named_groups = map_route(request.method, request.path)
    data = handler(request, *groups, **named_groups)
    data = ujson.dumps(data)

    status = '200'
    response_headers = [
        ('content-type', 'application/json',),
        ('content-length', str(len(data)),),
    ]

    start_response(status, response_headers)
    return iter([data])



import importlib
import os


def scan(root='.', exclusions=None):
    ''' root = directory tree root '''
    exclusions = exclusions or []
    exclusions = ['/'.join((root, val)) for val in exclusions]
    print exclusions
    for dir_name, subdir_list, file_list in os.walk(root):
        print 'dir=', dir_name
        # if dir_name in exclusions:
        #     continue
        for exc in exclusions:
            if 'scripts' in dir_name:
                print 'FIZZ: ', dir_name.startswith(exc)
            if dir_name.startswith(exc):
                continue
        check = (
            dir_name.startswith('./env') or
            dir_name.startswith('./.git') or
            '__init__.py' not in file_list
        )
        if check:
            continue
        for file_name in file_list:
            if not file_name.endswith('.py'):
                continue
            print 'in', dir_name, file_name
            if '__init__' in file_name:
                path = dir_name
                path = path[2:]
            else:
                path = '/'.join([dir_name, file_name])
                path = path[2:-3]  # trim ./ off front and .py off end
            path = path.replace('/', '.')
            importlib.import_module(path)

scan(exclusions=['scripts'])



from bedlam.app import run
from bedlam.decorators import route, get, post, put, delete
from bedlam.errors import BedlamError, NotFound, RequestError, PayloadError


__all__ = [
    'run',
    'route', 'get', 'post', 'put', 'delete',
    'BedlamError', 'NotFound', 'RequestError', 'PayloadError',
]



import re

from bedlam.errors import NotFound


_route_cache = {
    'get': [],
    'post': [],
    'put': [],
    'delete': [],
}


def register_route(method, uri_regex, handler):
    compiled = re.compile(uri_regex)
    _route_cache[method.lower()].append((compiled, handler,))


def map_route(method, uri):
    matches = _find_route_matches(method, uri)
    if not matches:
        raise NotFound

    match, handler = matches[0]

    groups = ()
    named_groups = match.groupdict()
    if not named_groups:
        groups = match.groups()
        named_groups = {}

    return handler, groups, named_groups


def _find_route_matches(method, uri):
    regexs_for_method = _route_cache[method.lower()]
    matches = []
    for regex, handler in regexs_for_method:
        match = regex.match(uri)
        if match:
            matches.append((match, handler,))
    return matches



from functools import partial

from bedlam.cache import register_route


def route(method, uri_pattern):

    def wrap(f):
        register_route(method, uri_pattern, f)
        return f

    return wrap


get = partial(route, 'GET')
post = partial(route, 'POST')
put = partial(route, 'PUT')
delete = partial(route, 'DELETE')



import urlparse

import ujson

from bedlam.errors import PayloadError, BedlamError


class Request(object):
    def __init__(self, environ):
        self._uri = environ['RAW_URI']
        self._path = environ['PATH_INFO'].strip('/')
        self._method = environ['REQUEST_METHOD']
        self._remote_addr = environ['REMOTE_ADDR']

        _query_string = environ['QUERY_STRING']
        self._query_params = QueryParams(_query_string)

        _wsgi_input = environ['wsgi.input']
        self._request_body = RequestBody(_wsgi_input)

    @property
    def uri(self):
        return self._uri

    @property
    def path(self):
        return self._path

    @property
    def method(self):
        return self._method

    @property
    def query_params(self):
        return self._query_params

    @property
    def body(self):
        return self._request_body


class QueryParams(object):
    def __init__(self, query_string):
        self._query_string = query_string
        query_params = urlparse.parse_qsl(query_string)
        self._query_params = dict(query_params)

    @property
    def raw(self):
        return self._query_string

    @property
    def as_dict(self):
        return self._query_params

    def __getitem__(self, key):
        return self._query_params.get(key)

    def __getattr__(self, key):
        return self._query_params.get(key)


class RequestBody(object):
    def __init__(self, wsgi_input):
        self._body = wsgi_input.read()
        self._deserialized = self._deserialize(self._body)

    @staticmethod
    def _deserialize(body):
        if not body:
            return {}

        try:
            body = ujson.loads(body)
        except ValueError as err:
            raise PayloadError(err)
        except:
            raise BedlamError

        if not body:
            return {}

        # if not isinstance(body, dict):
        #     raise PayloadError

        return body

    @property
    def raw(self):
        return self._body

    def __getitem__(self, key):
        return self._deserialized.get(key)

    def __getattr__(self, key):
        return self._deserialized.get(key)

    def __iter__(self):
        return self._deserialized.iteritems()
        # for k, v in self._deserialized.iteritems():
        #     yield k, v

    def __contains__(self, key):
        return key in self._deserialized



from bedlam.errors import PayloadError


def require(*keys):
    def wraps(f):
        def wrapper(req, *a, **kw):
            for k in keys:
                if k not in req.body:
                    raise PayloadError
            return f(req, *a, **kw)
        return wrapper
    return wraps


def allow(*keys):
    def wraps(f):
        def wrapper(req, *a, **kw):
            for k, _ in req.body:
                if k not in keys:
                    raise PayloadError
            return f(req, *a, **kw)
        return wrapper
    return wraps



from setuptools import find_packages, setup


# python setup.py sdist upload


long_desc = '''\

'''


if __name__ == '__main__':
    setup(
        packages=find_packages(),
        name='Bedlam',
        version='0.0.1',
        author='Christopher Sira',
        author_email='cbsira@gmail.com',
        license='BSD',
        url='https://github.com/csira/bedlam',
        description='',
        long_description=long_desc,
        install_requires=[
            'gunicorn',
            'ujson',
        ],
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Environment :: Web Environment',
            'Intended Audience :: Developers',
            'License :: Freely Distributable',
            'License :: OSI Approved :: BSD License',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.7',
            'Topic :: Internet :: WWW/HTTP',
            'Topic :: Internet :: WWW/HTTP :: HTTP Servers',
            'Topic :: Software Development :: Libraries :: Application Frameworks',
            'Topic :: Software Development :: Libraries :: Python Modules',
        ],
    )



class BedlamError(Exception):

    status = 500

    @classmethod
    def handle(cls, start_response):
        headers = [('content-length', '0')]
        start_response(str(cls.status), headers)
        return iter([''])


class NotFound(BedlamError):

    status = '404'


class RequestError(BedlamError):

    status = '400'


class PayloadError(RequestError):

    status = '400'
